/* astro1.c: astronomy sample code 1 for S2PLOT library
 *
 * David Barnes, January 2006
 *
 * $Id: astro1.c,v 3.4 2007/04/04 23:06:50 dbarnes Exp $
 *
 * This code loads HIPASS catalogue data from a comma-separated value
 * file.  It obtains RA, DEC, VEL_CMB and SINT fields.  Using standard
 * formulae, it calculates the HI mass of each object.  Then using
 * S2PLOT functions, it produces a 3d plot of the data, using VEL_CMB
 * to obtain a Hubble distance to each galaxy, and colouring the
 * galaxy according to its (log) estimated HI mass.  The selection
 * bias against low mass objects at high distances should be obvious.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "s2plot.h"

#define H0 75.0

#define MAPLO 2000
#define MAPHI  3000

#define D2R (M_PI / 180.)

#define INITMIN 9e30
#define INITMAX -9e30

#if defined(S2SUNOS)
#define log10f log10
#endif

int main(int argc, char **argv) {

  int i;

  FILE *fp = NULL;
  char string[255];
  float *ra = NULL;
  float *dec = NULL;
  float *vel = NULL;
  float *sint = NULL;
  float *loghi = NULL;
  int nvals = 0;

  float minvel = INITMIN;
  float maxvel = INITMAX;
  float minsint = INITMIN;
  float maxsint = INITMAX;
  float minloghi = INITMIN;
  float maxloghi = INITMAX;

  /* open data file */
  fp = fopen(argv[1], "r");
  if (!fp) {
    fprintf(stderr, "Cannot open file named on cmd-line.\n");
    exit(-1);
  }

  /* read data */
  while (!feof(fp) && fgets(string, 255, fp)) {
    if (string[0] == '#') {
      continue;
    } else {
      /* replace commas with white-space for sscanf to work! */
      for (i = 0; i < strlen(string); i++) {
	if (string[i] == ',') {
	  string[i] = ' ';
	}
      }
      float r1, d1, v1, s1;
      char i1[120];
      if (sscanf(string, "%f %f %s %f %f", &r1, &d1, i1, &v1, &s1) == 5) {
	/* we apparently read a line, so increase count and store data */
	nvals++;
	ra = (float *)realloc(ra, nvals*sizeof(float));
	dec = (float *)realloc(dec, nvals*sizeof(float));
	vel = (float *)realloc(vel, nvals*sizeof(float));
	sint = (float *)realloc(sint, nvals*sizeof(float));
	ra[nvals-1] = r1;
	dec[nvals-1] = d1;
	vel[nvals-1] = v1;
	sint[nvals-1] = s1;
      }
    }
  }
  fprintf(stderr, "# Read %d items from file %s\n", nvals, argv[1]);
  
  /* find data range and calculate HI mass */
  loghi = (float *)calloc(nvals, sizeof(float));
  for (i = 0; i < nvals; i++) {
    loghi[i] = log10f(2.356e5 * (vel[i] / H0) * (vel[i] / H0) * sint[i]);

    if (vel[i] < minvel) {
      minvel = vel[i];
    }
    if (vel[i] > maxvel) {
      maxvel = vel[i];
    }
    if (sint[i] < minsint) {
      minsint = sint[i];
    }
    if (sint[i] > maxsint) {
      maxsint = sint[i];
    }
    if (loghi[i] < minloghi) {
      minloghi = loghi[i];
    }
    if (loghi[i] > maxloghi) {
      maxloghi = loghi[i];
    }
  }

  /* some sensible constraints based on programmer's knowledge of
   * the database in use */
  if (minloghi < 6.) {
    minloghi = 6.;
  }
  if (maxloghi > 10.5) {
    maxloghi = 10.5;
  }

  fprintf(stderr, "# vel range = %f - %f\n", minvel, maxvel);
  fprintf(stderr, "# sint range = %f - %f\n", minsint, maxsint);
  fprintf(stderr, "# loghi range = %f - %f\n", minloghi, maxloghi);

  /* open S2PLOT device */
  if (!s2opend("/?", argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }
  s2swin(-1.,1.,-1.,1.,-1.,1.);
  //ss2sbc(1., 1., 1.);


  /* install a nice colormap */
  s2icm("rainbow", MAPLO, MAPHI);
  s2scir(MAPLO, MAPHI);

  /* loop and plot the points with 2-pixel blocks */
  s2slw(3);
  for (i = 0; i < nvals; i++) {
    if (vel[i] < 0) {
      continue;
    }
    int colidx;
    float x, y, z;

    colidx = MAPLO + (int)((float)(MAPHI - MAPLO) * (loghi[i] - minloghi) /
			   (maxloghi - minloghi) + 0.5);
    if (colidx < MAPLO) {
      colidx = MAPLO;
    }
    if (colidx > MAPHI) {
      colidx = MAPHI;
    }
    s2sci(colidx);

    x = cos(dec[i] * D2R) * cos(ra[i] * D2R);
    y = cos(dec[i] * D2R) * sin(ra[i] * D2R);
    z = sin(dec[i] * D2R);
    if (ss2qpt() < 2) {
      /* "cartesian" projection - place sources at redshift */
      x *= vel[i] / maxvel;
      y *= vel[i] / maxvel;
      z *= vel[i] / maxvel;
    }
    s2pt1(x, y, z, -2);
  }

  /* draw a hemispherical shell to give some context to the data */
  XYZ posn, norm, start;
  posn.x = 0.;
  posn.y = 0.;
  posn.z = 0.;
  norm.x = 0.;
  norm.y = 0.;
  norm.z = 1.;  
  s2slw(1);
  s2sci(S2_PG_DKGREY);
  /* lines of latitude */
  for (i = 0; i < 6; i++) {
    start.x = cos(((float)i * -15.) * D2R);
    start.y = 0.;
    start.z = sin(((float)i * -15.) * D2R);
    ns2varc(posn, norm, start, 360., 90);
    start.z = -start.z;
    ns2varc(posn, norm, start, 360., 90);
  }
  /* lines of longitude */
  norm.z = 0.;
  start.z = 0.;
  for (i = 0; i < 24; i++) {
    norm.x = -cos(((float)i * 15.) * D2R);
    norm.y = -sin(((float)i * 15.) * D2R);
    start.x = cos(((float)i * 15. + 90.) * D2R);
    start.y = sin(((float)i * 15. + 90.) * D2R);
    ns2varc(posn, norm, start, 360., 45);
  }

  /* now for testing for dome, I want to draw a circle on the "sky"
   * which is the unit sphere centred on the origin.  Propose doing
   * this by using ns2varc function, with posn = the origin, norm =
   * direction to centre of desired circle, and start = vector to 
   * a point on the circle (and thus on the "sky").
   */
  posn.x = posn.y = posn.z = 0.;
  /* centre of circle is 9h, -25 degrees */
  norm.x = cos(9. * 15. * D2R) * cos(-25. * D2R);
  norm.y = sin(9. * 15. * D2R) * cos(-25. * D2R);
  norm.z = sin(-25. * D2R);
  /* a position on the circle is 9h, -32 degrees for radius = 7 degrees */
  start.x = cos(9. * 15. * D2R) * cos(-32. * D2R);
  start.y = sin(9. * 15. * D2R) * cos(-32. * D2R);
  start.z = sin(-32. * D2R);
  s2sci(S2_PG_CYAN);
  s2slw(2);
  ns2varc(posn, norm, start, 360., 60);


  /* if we are in a dome ... */
  if (ss2qpt() == 2) {
    /* set the camera position to the origin, looking at the pole */
    XYZ position = {0., 0., 0.};
    XYZ up = {0., 1., 0.};
    XYZ vdir = {0., 0., -1.};
    ss2sc(position, up, vdir, 0);

    /* disallow translations of the camera */
    ss2tc(0);

    /* rotate the fisheye to place the pole 52 deg. up from horizon */
    /* (52deg = 90deg - approx latitude of Swinburne) :-) */
    ss2sfra(52.0);

  }

  /* points and axes drawn, so enter interactive display mode */
  s2show(1);

  return 0;
}
