/* astro1a.c: astronomy sample code 1a for S2PLOT library
 *
 * David Barnes, January 2006
 *
 * $Id: astro1a.c,v 3.5 2007/04/04 23:06:50 dbarnes Exp $
 *
 * This code loads HIPASS catalogue data from a comma-separated value
 * file.  It obtains RA, DEC, VEL_CMB and SINT fields.  Using standard
 * formulae, it calculates the HI mass of each object.  Then using
 * S2PLOT functions, it produces a 3d plot of the data.  Galaxies are
 * drawn in a point size proportional to the log of their HI mass.
 * Galaxies are actually drawn in a dynamic callback, which considers
 * the toggle state of the numeric keys to select which velocity ranges
 * of galaxies to plot.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "s2plot.h"

#define H0 75.0

#define MAPLO 2000
#define MAPHI  3000

#define D2R (M_PI / 180.)

#define INITMIN 9e30
#define INITMAX -9e30

#if defined(S2SUNOS)
#define log10f log10
#endif

void numcb(int *num);
void hicb(double *time, int *keycount);

char fname[255];

int main(int argc, char **argv) {

  if (argc < 2) {
    fprintf(stderr, "please provide data file name on c/line.\n");
    exit(-1);
  }
  strcpy(fname, argv[1]);

  /* open S2PLOT device */
  if (!s2opend("/?", argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }
  s2swin(-1.,1.,-1.,1.,-1.,1.);
  ss2sbc(1., 1., 1.);

  /* install a nice colormap */
  /* next two lines give figures in PASA paper */
  s2icm("grey", MAPLO, MAPHI);
  s2scir(MAPLO, MAPLO + (MAPHI-MAPLO)/2);
  /* next two lines give colour figures for web pages */
  //s2icm("rainbow", MAPLO, MAPHI);
  //s2scir(MAPLO, MAPHI);


  /* draw a hemispherical shell to give some context to the data */
  XYZ posn, norm, start;
  int i;
  posn.x = 0.;
  posn.y = 0.;
  posn.z = 0.;
  norm.x = 0.;
  norm.y = 0.;
  norm.z = 1.;  
  s2slw(1);
  s2sci(S2_PG_DKGREY);
  /* lines of latitude */
  for (i = 0; i < 6; i++) {
    start.x = cos(((float)i * -15.) * D2R);
    start.y = 0.;
    start.z = sin(((float)i * -15.) * D2R);
    ns2varc(posn, norm, start, 360., 90);
    start.z = -start.z;
    ns2varc(posn, norm, start, 360., 90);
  }
  /* lines of longitude */
  norm.z = 0.;
  start.z = 0.;
  for (i = 0; i < 24; i++) {
    norm.x = -cos(((float)i * 15.) * D2R);
    norm.y = -sin(((float)i * 15.) * D2R);
    start.x = cos(((float)i * 15. + 90.) * D2R);
    start.y = sin(((float)i * 15. + 90.) * D2R);
    ns2varc(posn, norm, start, 360., 45);
  }

  /* draw circle around Centaurus */
  posn.x = posn.y = posn.z = 0.;
  /* centre of circle is 12h40, -41 degrees */
  norm.x = cos(12.66 * 15. * D2R) * cos(-41. * D2R);
  norm.y = sin(12.66 * 15. * D2R) * cos(-41. * D2R);
  norm.z = sin(-41. * D2R);
  /* a position on the circle is 12h40, -31 degrees for radius = 10 degrees */
  start.x = cos(12.66 * 15. * D2R) * cos(-31. * D2R);
  start.y = sin(12.66 * 15. * D2R) * cos(-31. * D2R);
  start.z = sin(-31. * D2R);
  s2sci(S2_PG_BLACK);
  s2slw(2);
  ns2varc(posn, norm, start, 360., 60);

  /* draw circle around Fornax */
  posn.x = posn.y = posn.z = 0.;
  /* centre of circle is 3h40, -35.5 degrees */
  norm.x = cos(3.66 * 15. * D2R) * cos(-35.5 * D2R);
  norm.y = sin(3.66 * 15. * D2R) * cos(-35.5 * D2R);
  norm.z = sin(-35.5 * D2R);
  /* a position on the circle is 3h40, -27.5 degrees for radius = 8 degrees */
  start.x = cos(3.66 * 15. * D2R) * cos(-27.5 * D2R);
  start.y = sin(3.66 * 15. * D2R) * cos(-27.5 * D2R);
  start.z = sin(-27.5 * D2R);
  s2sci(S2_PG_BLACK);
  s2slw(2);
  ns2varc(posn, norm, start, 360., 60);


  /* if we are in a dome ... */
  if (ss2qpt() == 2) {
    /* set the camera position to the origin, looking at the pole */
    XYZ position = {0., 0., 0.};
    XYZ up = {0., 1., 0.};
    XYZ vdir = {0., 0., -1.};
    ss2sc(position, up, vdir, 0);

    /* disallow translations of the camera */
    ss2tc(0);

    /* rotate the fisheye to place the pole 52 deg. up from horizon */
    /* (52deg = 90deg - approx latitude of Swinburne) :-) */
    //ss2sfra(52.0);

  }

  /* install the callback to draw the catalogue sources */
  cs2scb(hicb);

  /* install the callback to handle numeric key presses */
  cs2sncb(numcb);

  /* points and axes drawn, so enter interactive display mode */
  s2show(1);

  return 0;
}

int keymask[10] = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1};
void numcb(int *num) {
  if ((*num >= 0) && (*num <= 9)) {
    keymask[(*num + 9) % 10] = !keymask[(*num + 9) % 10];
  }
}

void hicb(double *time, int *keycount) {
  static int beenhere = 0;
  static float *ra = NULL;
  static float *dec = NULL;
  static float *vel = NULL;
  static float *sint = NULL;
  static float *loghi = NULL;
  static int nvals = 0;
  static float minvel = INITMIN;
  static float maxvel = INITMAX;
  static float minsint = INITMIN;
  static float maxsint = INITMAX;
  static float minloghi = INITMIN;
  static float maxloghi = INITMAX;
  static int kcounter;

  int i;

  if (!beenhere) {
    /* first time through - initialise */

    kcounter = *keycount;

    FILE *fp = NULL;
    char string[255];

    /* open data file */
    fp = fopen(fname, "r");
    if (!fp) {
      fprintf(stderr, "Cannot open file named on cmd-line %s.\n", fname);
      exit(-1);
    }
    
    /* read data */
    while (!feof(fp) && fgets(string, 255, fp)) {
      if (string[0] == '#') {
	continue;
      } else {
	/* replace commas with white-space for sscanf to work! */
	for (i = 0; i < strlen(string); i++) {
	  if (string[i] == ',') {
	    string[i] = ' ';
	  }
	}
	float r1, d1, v1, s1;
	char i1[120];
	if (sscanf(string, "%f %f %s %f %f", &r1, &d1, i1, &v1, &s1) == 5) {
	  /* we apparently read a line, so increase count and store data */
	  nvals++;
	  ra = (float *)realloc(ra, nvals*sizeof(float));
	  dec = (float *)realloc(dec, nvals*sizeof(float));
	  vel = (float *)realloc(vel, nvals*sizeof(float));
	  sint = (float *)realloc(sint, nvals*sizeof(float));
	  ra[nvals-1] = r1;
	  dec[nvals-1] = d1;
	  vel[nvals-1] = v1;
	  sint[nvals-1] = s1;
	}
      }
    }
    fclose(fp);
    fprintf(stderr, "# Read %d items from file %s\n", nvals, fname);
    
    /* find data range and calculate HI mass */
    loghi = (float *)calloc(nvals, sizeof(float));
    for (i = 0; i < nvals; i++) {
      loghi[i] = log10f(2.356e5 * (vel[i] / H0) * (vel[i] / H0) * sint[i]);
      
      if (vel[i] < minvel) {
	minvel = vel[i];
      }
      if (vel[i] > maxvel) {
	maxvel = vel[i];
      }
      if (sint[i] < minsint) {
	minsint = sint[i];
      }
      if (sint[i] > maxsint) {
	maxsint = sint[i];
      }
      if (loghi[i] < minloghi) {
	minloghi = loghi[i];
      }
      if (loghi[i] > maxloghi) {
	maxloghi = loghi[i];
      }
    }
    
    /* some sensible constraints based on programmer's knowledge of
     * the database in use */
    if (minloghi < 6.) {
      minloghi = 6.;
    }
    if (maxloghi > 10.5) {
      maxloghi = 10.5;
    }

    fprintf(stderr, "# vel range = %f - %f\n", minvel, maxvel);
    fprintf(stderr, "# sint range = %f - %f\n", minsint, maxsint);
    fprintf(stderr, "# loghi range = %f - %f\n", minloghi, maxloghi);
    
    /* and never initialise again! */
    beenhere = 1;
  }

  /* get the current color index range */
  int clo, chi;
  s2qcir(&clo, &chi);

  /* loop and plot the points */
  for (i = 0; i < nvals; i++) {
    if (vel[i] < 0) {
      continue;
    }
    if (vel[i] > 9999.99) {
      continue;
    }
    if (loghi[i] < 5) {
      continue;
    }
    
    /* is this velocity group supposed to be plotted? */
    int key = (int)(vel[i] / 1000);
    if (!keymask[key]) {
      continue;
    }

    int colidx;
    float wid;
    float x, y, z;
    float r, g, b;

    colidx = clo + (int)((float)(chi - clo) * (vel[i] - minvel) /
			 (maxvel - minvel) + 0.5);
    if (colidx < clo) {
      colidx = clo;
    }
    if (colidx > chi) {
      colidx = chi;
    }
    s2sci(colidx);
    s2qcr(colidx, &r, &g, &b);
    
    wid = (loghi[i] - 4.) * 1.2;

    x = cos(dec[i] * D2R) * cos(ra[i] * D2R);
    y = cos(dec[i] * D2R) * sin(ra[i] * D2R);
    z = sin(dec[i] * D2R);
    if (ss2qpt() < 2) {
      /* "cartesian" projection - place sources at redshift */
      x *= vel[i] / maxvel;
      y *= vel[i] / maxvel;
      z *= vel[i] / maxvel;
    }
    ns2thpoint(x, y, z, r, g, b, wid);
  }
  
  kcounter = *keycount;

}
