/* isosurf.c: demonstration of isosurfacing in S2PLOT
 *
 * David Barnes, February 2007
 *
 * $Id: isosurf.c,v 1.2 2007/04/04 22:56:39 dbarnes Exp $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "s2plot.h"

#define BWIDTH 30
int nx, ny, nz;
float ***grid;
float tr[12];
float level;
int id3;

void callback(double *time, int *keycount);
void colorfn(float *x, float *y, float *z, 
	     float *r, float *g, float *b);

int main(int argc, char **argv) {

  /* check usage */
  if (argc != 1) {
    fprintf(stderr, "usage: %s\n", argv[0]);
    exit(-1);
  }

  /* grid dimensions */
  nx = BWIDTH;
  ny = BWIDTH;
  nz = BWIDTH;

  /* transpose matrix mapping data indices to world coords */
  /* offsets */
  tr[0] = tr[4] = tr[8] = 0.0;
  /* increments */
  tr[1] = tr[6] = tr[11] = 1.0;
  /* cross terms */
  tr[2] = tr[3] = tr[5] = tr[7] = tr[9] = tr[10] = 0.;

  /* open S2PLOT device */
  if (!s2opend("/?", argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }

  /* set up coordinate system */
  s2svp(-0.7, 0.7, -0.7, 0.7, -0.7, 0.7);
  s2swin(0, nx-1, 0, ny-1, 0, nz-1);
  char *opt = "BCDETSMNQ";

  /* allocate and generate the data grid */
  grid = (float ***)malloc(nx * sizeof(float **));
  int i, j, k;
  float x, y, z;
  for (i = 0; i < nx; i++) {
    grid[i] = (float **)malloc(ny * sizeof(float *));
    x = (float)(i) / (float)(nx - 1);
    for (j = 0; j < ny; j++) {
      grid[i][j] = (float *)malloc(nz * sizeof(float));
      y = (float)(j) / (float)(ny - 1);
      for (k = 0; k < nz; k++) {
	z = (float)(k) / (float)(nz - 1);
	grid[i][j][k] = x*x*x + y*y - z*z*z*z;
      }
    }
  }
  
  /* draw and label a box around the points, in a non-green-ish colour */
  s2sci(S2_PG_RED);
  s2sch(1.2);
  s2slw(1);
  s2box(opt, 0.0, 0, opt, 0.0, 0, opt, 0.0, 0);
  s2lab("x", "y", "z", "isosurf demo");

  int resolution = 1;

  /* create and draw first (yellow) isosurface */
  int id1 = ns2cis(grid, nx, ny, nz, 0, nx-1, 0, ny-1, 0, nz-1,
		   tr, 0.0, resolution, 
		   'o', 1., 1., 1., 0.);
  ns2dis(id1, 0);

  /* create and draw second (coloured by function) isosurface */
  int id2 = ns2cisc(grid, nx, ny, nz, 0, nx-1, 0, ny-1, 0, nz-1,
		    tr, 1.0, resolution, 
		    's', 0.8, colorfn);
  ns2dis(id2, 0);

  /* create BUT DO NOT DRAW third isosurface */
  id3 = ns2cis(grid, nx, ny, nz, nx/2, nx-1, 0, ny/2, nz/3, 2*nz/3,
	       tr, 0.5, resolution, 's', 0.6,
	       0.7, 0.3, 0.2);
  
  /* register callback */
  cs2scb(callback);

  /* show the environment */
  s2show(1);

  return (0);
}


void callback(double *time, int *keycount) {
  
  static int lastkeycount = -1;
  if (lastkeycount < 0) {
    lastkeycount = *keycount;
  }

  level = (*keycount % 9 + 1) / 10.;
  if (lastkeycount != *keycount) {
    fprintf(stderr, "isosurface level changed to %f\n", level);
  }
  lastkeycount = *keycount;

  /* change the isosurface level */
  ns2sisl(id3, level);

  /* and draw it */
  ns2dis(id3, 0);

}

void colorfn(float *x, float *y, float *z, 
	     float *r, float *g, float *b) {
  *r = 0.2 + 0.5 * (*x) / (float)(nx - 1);
  *g = 0.0 + 0.8 * (*y) / (float)(ny - 1);
  *b = 0.6 + 0.2 * (*z) / (float)(nz - 1);
}

