/* s2uc4.c: demonstration / use case 4 for S2PLOT library.
 *
 * David Barnes, October 2005
 *
 * $Id: s2uc4.c,v 3.1 2006/12/22 03:33:25 dbarnes Exp $
 *
 * Demonstrates logarithmic and exponential/decimal labelling options, 
 * and 3-d vector field plotting.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "s2plot.h"

#define XLIMIT 2.2
#define YLIMIT 3.4
#define ZLIMIT 1.6
#define TWOPI 6.2831853

int main(int argc, char **argv) {

  int fullscreen = 0; /* not full-screen mode */
  int stereo = 2;     /* mono mode (1 for active stereo, 2 for passive) */

  float x1 = -XLIMIT, x2 = XLIMIT; /* x world coord range */
  float y1 = -YLIMIT, y2 = YLIMIT; /* y world coord range */
  float z1 = ZLIMIT+4.0, z2 = 3.0 * ZLIMIT + 4.0; /* z world coord range */

  char *xopt = "BCDETSGMNL";           /* B = draw front bottom (-Y,-Z) edge 
				    * C = draw front top (-Y,+Z) edge
				    * D = draw back bottom (+Y,-Z) edge
				    * E = draw back top (+Y,+Z) edge
				    * T = draw major tick marks
				    * S = draw minor tick marks
				    */
  char *yopt = "BCDETSGMN";           /* B = draw bottom left (-X,-Z) edge
				    * C = draw bottom right (+X,-Z) edge
				    * D = draw top left (-X,+Z) edge
				    * E = draw top right (+X,+Z) edge
				    */
  char *zopt = "BCDETSGMNL2";           /* B = draw left front (-X,-Y) edge
				    * C = draw right front (+X,-Y) edge
				    * D = draw left back (-X,+Y) edge
				    * E = draw right back (+X,+Y) edge
				    */

  /* major tick increments on all axes */
  float xtick = 0.0, ytick = 0.0, ztick = 0.0;

  /* number of subdivisions on all axes */
  int nxsub = 4, nysub = 1, nzsub = 10;

  int i, j, k; /* loop variable/s */

  int interactive = 1; /* 0 = just draw; 1 = draw and go interactive */

  /* open S2PLOT device */
  if (!s2open(fullscreen, stereo, argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }

  /* set world coordinate range */
  s2svp(-0.5, 0.5, -0.5, 0.5, -0.5, 0.5);
  s2swin(x1, x2, y1, y2, z1, z2);

  /* Let's start by generating a vector set to work with. */
  #define NXI 10
  #define NYI 10
  #define NZI 15
  float ***a, ***b, ***c;
  float minlength = 9e30;
  float maxlength = -9e30;
  float length;
  float tmpx, tmpy, tmpz;

  a = (float ***)calloc(NXI, sizeof(float ***));
  b = (float ***)calloc(NXI, sizeof(float ***));
  c = (float ***)calloc(NXI, sizeof(float ***));
  for (i = 0; i < NXI; i++) {
    a[i] = (float **)calloc(NYI, sizeof(float **));
    b[i] = (float **)calloc(NYI, sizeof(float **));
    c[i] = (float **)calloc(NYI, sizeof(float **));
    tmpx = (float)i / (float)(NXI-1) * TWOPI;
    for (j = 0; j < NYI; j++) {
      a[i][j] = (float *)calloc(NZI, sizeof(float *));
      b[i][j] = (float *)calloc(NZI, sizeof(float *));
      c[i][j] = (float *)calloc(NZI, sizeof(float *));
      tmpy = (float)j / (float)(NYI-1) * 2.0 * TWOPI;
      for (k = 0; k < NZI; k++) {
	tmpz = (float)k / (float)(NZI-1) * 4.0 * TWOPI;

	a[i][j][k] = sin(tmpx) + cos(tmpy) * tmpz / 14.0;
	b[i][j][k] = cos(tmpx) - sin(tmpy) * tmpz / 13.0;
	c[i][j][k] = 0.5 + sin(tmpx) * cos(tmpy);

	length = sqrt(a[i][j][k] * a[i][j][k] +
		      b[i][j][k] * b[i][j][k] +
		      c[i][j][k] * c[i][j][k]);
	if (length > maxlength) {
	  maxlength = length;
	}
	if (length < minlength) {
	  minlength = length;
	}
      }
    }
  }

  /* set up a unity transformation matrix */
  float tr[12];
  tr[0] = x1; tr[1] = (x2-x1)/(float)(NXI-1); tr[2] = 0.0; tr[3] = 0.0;
  tr[4] = y1; tr[5] = 0.0; tr[6] = (y2-y1)/(float)(NYI-1); tr[7] = 0.0;
  tr[8] = z1; tr[9] = 0.0; tr[10]= 0.0; tr[11] = (z2-z1)/(float)(NZI-1);

  /* install a cool colormap */
  s2icm("gray", 1000, 1400);
  s2scir(1000, 1400);

  /* plot vectors */
  s2sah(1, 30.0, 0.4);
  s2vect3(a, b, c, NXI, NYI, NZI, 0, NXI-1, 0, NYI-1, 0, NZI-1,
	  0.3, 0, tr, -1, 1, minlength, maxlength);


  /* draw and label a box around the points, in a non-green-ish colour */
  s2sci(S2_PG_RED);
  s2sch(1.2);
  s2slw(1);
  s2box(xopt, xtick, nxsub, yopt, ytick, nysub, zopt, ztick, nzsub);

  /* flush graphics and enter interactive mode */
  s2show(interactive);

  /* successful run */
  return 0;

}
