/* s2uc5.c: demonstration use case 5 for S2PLOT library.
 *
 * David Barnes, October 2005
 *
 * $Id: s2uc5.c,v 3.3 2007/04/04 22:55:26 dbarnes Exp $
 *
 * Demonstrates the "native" S2PLOT functions.  These functions are
 * the most directly related to the GEOM format description by Paul
 * Bourke and supported by the stereo2 and geomviewer programs.
 * The general use of these functions is to use textures, and/or to 
 * use some internal features such as colour blending along lines 
 * and on faces.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "s2plot.h"

#define XLIMIT 10.0
#define YLIMIT 10.0
#define ZLIMIT 10.0

int main(int argc, char **argv) {

  /* open S2PLOT device using env var or prompting */
  if (!s2opend("/?", argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }

  float x1 = -XLIMIT, x2 = XLIMIT; /* x world coord range */
  float y1 = -YLIMIT, y2 = YLIMIT; /* y world coord range */
  float z1 = -ZLIMIT, z2 = ZLIMIT; /* z world coord range */

  /* set world coordinate range */
  s2swin(x1, x2, y1, y2, z1, z2);

  /* draw and label a box around the points, in white */
  s2sci(S2_PG_WHITE);
  s2sch(1.2);
  s2slw(1);
  s2box("BCDETS", 0., 0, "BCDETS", 0., 0, "BCDETS", 0., 0);

 

  /* native spheres */
  {
    ns2sphere(-2., 2., 2., 0.5, 0.6, 0.6, 0.9);
    XYZ position = {-4., 2., 2.};
    COLOUR colour = {0.9, 0.6, 0.6};
    ns2vsphere(position, 0.75, colour);
  }

  /* native textured spheres */
  {
    ns2spheret(2., -2., 2., 0.5, 0.6, 0.6, 0.9, "firetile2_pow2_rgb.tga");
    XYZ position = {2., -4., 2.};
    COLOUR colour = {0.9, 0.6, 0.6};
    ns2vspheret(position, 0.75, colour, "firetile2_pow2_rgb.tga");
  }

  /* native disks */
  {
    ns2disk(2., 2., 2., -1., -1., -1., 1., 2., 0.8, 0.4, 0.2);
    XYZ position = {2., 2., 2.};
    XYZ normal = {-1., 1., 0.5};
    COLOUR colour = {0.4, 0.2, 0.8};
    ns2vdisk(position, normal, 0.0, 2.0, colour);
  }

  /* native labels */
  {
    ns2text(0., 0., 0.,    1., 0., 0.,   0., 1., 0., 
	    0.2, 0.8, 0.7, "README.TXT");
    XYZ position = {0., 2., 0.};
    XYZ right = {1., 0., 0.};
    XYZ up = {0., 0.71, 0.71};
    COLOUR colour = {0.7, 0.8, 0.2};
    ns2vtext(position, right, up, colour, "METOO.FITS");
  }

  /* native points */
  {
    ns2point(-3., -3., -3., 1., 0., 0.);
    XYZ position = {-3., -3., -3.5};
    COLOUR colour = {0., 1., 0.};
    ns2vpoint(position, colour);
  }

  /* native thick points */
  {
    ns2thpoint(-3., -3., -4., 1., 1., 1., 4.);
    XYZ position = {-3., -3., -4.5};
    COLOUR colour = {0.2, 0.5, 1.0};
    ns2vthpoint(position, colour, 8.);
  }

  /* native lines */
  {
    ns2line(4., 4., -4.,   5., 5., -4., 0.8, 0.2, 0.8);
    XYZ p1 = {4., 4., -4.};
    XYZ p2 = {5., 5., -4.5};
    COLOUR colour = {0.8, 0.5, 0.8};
    ns2vline(p1, p2, colour);
  }

  /* native thick lines */
  { 
    ns2thline(4., 4., -4.5, 5., 5., -4.5, 0.8, 0.2, 0.8, 3.0);
    XYZ p1 = {4., 4., -4.5};
    XYZ p2 = {5., 5., -5.0};
    COLOUR colour = {0.8, 0.5, 0.8};
    ns2vthline(p1, p2, colour, 3.0);
  }

  /* native colour-blended lines */
  {
    ns2cline(4., 4., -5.,    6., 6., -5., 
	     0.8, 0.2, 0.8,  0.2, 0.8, 0.2);
    XYZ p1 = {4., 4., -5.5};
    XYZ p2 = {6., 6., -6.0};
    COLOUR col1 = {1., 0., 0.};
    COLOUR col2 = {0., 1., 1.};
    ns2vcline(p1, p2, col1, col2);
  }
  
  /* 3-vertex facets */
  {
    XYZ p[3] = {{-6., -6., 6.}, {-6., -8., 6.}, {-7., -5., 7.}};
    COLOUR col = {0.8, 0.4, 0.2};
    ns2vf3(p, col);

    XYZ pp[3] = {{-9., -6., 6.}, {-9., -8., 6.}, {-9., -5., 7.}};
    COLOUR cols[3] = {{1., 0., 0.}, {0., 1., 0.}, {0., 0., 1.}};
    ns2vf3c(pp, cols);
    
    XYZ ppp[3] = {{-9., -1., 6.}, {-9., -3., 6.}, {-9., 0., 7.}};
    XYZ n[3] = {{1., 1., 0.}, {1., 0., 1.}, {0., 1., 1.}};
    ns2vf3n(ppp, n, cols[1]);
    
    XYZ pppp[3] = {{-9., -7., 4.}, {-9., -11., 6.}, {-7., -5., 7.}};
    ns2vf3nc(pppp, n, cols);
  }
  
  /* 4-vertex facets */
  {
    XYZ p[4] = {{6., 6., 6.}, {6., 7., 4.}, {7., 7., 4.}, {7., 6., 6.}};
    COLOUR col = {0.8, 0.4, 0.2};
    ns2vf4(p, col);

    XYZ pp[4] = {{6., 6., 7.}, {6., 7., 7.}, {7., 7., 8.}, {7., 6., 8.}};
    COLOUR cols[4] = {{1., 1., 0.}, {0., 1., 1.}, {1., 0., 1.}, 
		      {0.5, 0.5, 0.5}};
    ns2vf4c(pp, cols);
    
    XYZ ppp[4] = {{6., 6., 9.}, {6., 5., 9.}, {5., 5., 9.}, {5., 6., 9.}};
    XYZ n[4] = {{0., 0., 1.}, {0., 1., 1.}, {1., 0., 1.}, {1., 1., 1.}};
    ns2vf4n(ppp, n, cols[1]);
    
    /* ns2vf4nc UNTESTED */
  }


  /* textured 4-vertex facet */
  {
    XYZ p[4] = {{3., -3., -8.}, 
		{5., -3., -8.},
		{5., -5., -9.},
		{3., -5., -9.}};
    COLOUR col = {1., 1., 1.};
    ns2vf4t(p, col, "strongbluemarble_pow2_rgb.tga", 1.0, 'o');
  }

  /* two lights */
  { 
    /* a slightly blue one near the textured 4-vertex facet */
    ns2i(4., -4., -11., 0.7, 0.7, 1.);
    /* a white one somewhere else! */
    XYZ p = {-4., 4., 11.};
    COLOUR col = {1., 1., 1.};
    ns2vi(p, col);
  }

  /* markers */
  {
    ns2m(0, 1., 4., 8., -1.5, 0.6, 0.2, 0.3);
    ns2m(1, 2., 4., 8., -3., 0.6, 0.3, 0.2);
    XYZ p = {4., 8., -4.5};
    COLOUR col = {0.2, 0.6, 0.3};
    ns2vm(2, 0.8, p, col);
    p.z = -6.;
    ns2vm(3, 1.6, p, col);
  }

  /* flush graphics and enter interactive mode */
  s2show(1);

  //fprintf(stderr, "control returned to main!!!\n");

  /* successful run */
  return 0;

}
