/* s2uc7.c: demonstration / use case 7 for S2PLOT library.
 *
 * David Barnes, October 2005
 *
 * $Id: s2uc7.c,v 3.12 2007/04/04 23:07:00 dbarnes Exp $
 *
 * Demonstrates "complex" dynamic geometry in the form of a surface 
 * plot which represents waves moving in the x and y directions as
 * a function of time, and a sphere which "floats" on the wave/s.  
 * Also demonstrates surface plotting where the range (ie. datamin 
 * and datamax) are given explicitly for colour map purposes.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "s2plot.h"
#include "string.h"

#if defined(S2LINUX) || defined(S2SUNOS)
#include <GL/glut.h>
#endif
#if defined(S2DARWIN)
#include <GLUT/glut.h>
#endif

#define XLIMIT 10.0
#define YLIMIT 20.0
#define TWOPI 6.2831853

/* define f(x,y,[t]), using a global variable _phase to pass [t] to
 * the function.
 */
float _phase;
float wave(float *x, float *y) {
  static float x1den = 1.0 / (XLIMIT - -XLIMIT);
  static float y1den = 1.0 / (YLIMIT - -YLIMIT);
  float x1 = (*x - -XLIMIT) * x1den;
  float y1 = (*y - -YLIMIT) * y1den;
  return ( cos((x1 * 0.5 + 0.5 * _phase) * TWOPI) +
	   sin((y1 * 1.3 + 0.25 * _phase) * TWOPI));
}

/* define an S2PLOT callback function which sets the global variable
 * _phase, draws the surface then a few extras.
 */
void newcallback(double *time, int *keycount) {
  /* textureID */
  static unsigned int textureID1, textureID2;
  static int textureloaded = 0;

  /* load the texture we will use again and again */
  if (!textureloaded) {
    textureID1 = ss2lt("firetile2_pow2_rgb.tga");
    textureID2 = ss2lt("strongbluemarble_pow2_rgb.tga");
    textureloaded = 1;
  }

  static XYZ P[4] = {{-XLIMIT, -YLIMIT, -2.}, 
		     {-XLIMIT,  YLIMIT, -2.},
		     { XLIMIT,  YLIMIT, -2.},
		     { XLIMIT, -YLIMIT, -2.}};
  static COLOUR col = {1., 1., 1.};

  /* phase is between 0 and 4 seconds */
  _phase = *time - (int)(*time / 4.0) * 4.0;

  /* draw the wave */
  s2funxyr(wave, 20, 20, -XLIMIT, XLIMIT, -YLIMIT, YLIMIT, 0, -2.0, 2.0);

  /* draw a ball "floating" on the wave */
  s2sci((*keycount % 15) + 1);
  s2sch(2.0);
  float atmp = 0.0;
  ss2ssr(32);

  if (*keycount % 2) {

    ns2spherex(0.0, 0.0, wave(&atmp, &atmp) + 0.4, 0.5, 1., 1., 1.,
		textureID1);

    /* for fun: fiddle with the second texture map */

    int w, h;
    char *tmap = ss2gt(textureID2, &w, &h);

    char *block = (char *)calloc(w * h * 4, sizeof(char));
    memcpy(block, tmap, w * 4 * sizeof(char));
    memmove(tmap, tmap + w * 4 * sizeof(char), w*(h-1)*4*sizeof(char));
    memcpy(tmap + w * (h-1) * 4 * sizeof(char), block, w * 4 * sizeof(char));
    free(block);

    ss2pt(textureID2);
    ns2vf4x(P, col, textureID2, 1.0, 'o');

  } else {
    s2pt1(0.0, 0.0, wave(&atmp, &atmp) + 0.2, 4);
  }

  float x1, x2, y1, y2, z1, z2;
  s2qwin(&x1, &x2, &y1, &y2, &z1, &z2);

  /* draw a string holding the ball vertically */
  float x[2], y[2], z[2];
  x[0] = x[1] = 0.0;
  y[0] = y[1] = 0.0;
  z[0] = wave(&atmp, &atmp) + 0.2;
  z[1] = z2;
  s2line(2, x, y, z);

  /* and axes in the "top" xy plane where the "string" is based */
  z[0] = z[1] = z2;
  x[0] = x1;
  x[1] = x2;
  y[0] = y[1] = 0.0;
  s2line(2, x, y, z);
  x[0] = x[1] = 0.0;
  y[0] = y1;
  y[1] = y2;
  s2line(2, x, y, z);
}


/* an opengl callback */
void openglcb(void) {
#if !defined(S2CYGWIN)
  glEnable(GL_BLEND);
  glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);

  glBegin(GL_QUADS);
  glColor4f(0.8, 0.3, 0.0, 0.5);
  glVertex3f(-3., -3., -3.);
  glVertex3f(-3.,  3., -3.);
  glVertex3f( 3.,  3., -3.);
  glVertex3f( 3., -3., -3.);
  glEnd();

  glDisable(GL_BLEND);
#endif
}


/* define the main function, which just installs a colormap for the 
 * surface plot, configures the S2PLOT coordinate system, draws and
 * labels the coordinates, draws a wedge, and then registers the 
 * callback function and "goes interactive".
 */
int main(int argc, char **argv) {
  /* open S2PLOT device using env var or prompting */
  if (!s2opend("/?", argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }
  
  /* install a very nice colormap */
  s2icm("astro", 1000, 1400);
  s2scir(1000, 1400);

  /* set up coordinate system and draw and label the coordinates */
  s2sci(S2_PG_WHITE);
  s2swin(-XLIMIT, XLIMIT, -YLIMIT, YLIMIT, -2.0, 2.0);
  s2box("BCDETSMNQ", 0., 0, "BCDETSMNQ", 0., 0, "BCDETSMNQ", 0., 0);
  s2lab("x", "y", "f(x,y,t)", "s2funxy used to plot the surface z=f(x,y,t)");

  s2wedge("Jy", 0.03, 0.06, 0.03, 0.41, -2.0, 2.0, S2_PG_WHITE);

  /* install a callback function to plot the dynamic surface geometry */
  cs2scb(&newcallback);

  /* install a callback function to draw direct OpenGL content */
  cs2socb(openglcb);

  /* flush graphics and enter interactive mode */
  s2disp(-1, 1); /* never timeout */

  // NOTE: control only flows beyond this point for Apple/OSX installations
  // using genuine and up-to-date Apple GLUT, or GNU/Linux installations
  // using glut with mods.

  fprintf(stderr, "control returned to main ... re-show\n");

  s2eras();
  s2sci(S2_PG_RED);
  s2box("BCDEG", 0., 0, "BCDEG", 0., 0, "BCDEG", 0., 0);

  s2disp(10, 0); /* 10 second timeout */

  s2icm("rainbow", 1401, 1800);
  s2scir(1401, 1800);

  /* control the camera */
  XYZ posn = {5., 5., 5.};
  XYZ up   = {5., 5., -5.};
  XYZ vdir = {-1., -1., -1.};
  ss2sc(posn, up, vdir, 1); /* set camera position in world corods */
  ss2sas(0); /* stop rotation if any */

  s2disp(10, 0); /* 10 second timeout */

  /* successful run */
  return 0;
}
