/* xastro1.c: extended astronomy sample code 1 for S2PLOT library
 *
 * David Barnes, January 2006
 *
 * $Id: xastro1.c,v 3.5 2007/04/04 23:07:00 dbarnes Exp $
 *
 * This code uses coordinate transformation routines to place the
 * Sun, the Moon and the constellation Crux on a spherical surface
 * (a "sky") at intervals over a 24-hr period.  It explicitly uses
 * a fisheye-type projection.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "s2plot.h"

void skycb(double *time, int *keycount);


#include "coco.h"
extern int kinit(void);

/* the constellation Crux */
float crux[5][7] = {{12,26,35.9,-1,-63,05,56}, /* -1 is sign value */
		    {12,47,43.3,-1,-59,41,19},
		    {12,31,09.9,-1,-57,06,47},
		    {12,15,08.6,-1,-58,44,56},
		    {12,21,21.5,-1,-60,24,04}};

#define D2R (M_PI / 180.)

int main(int argc, char **argv) {

  /* open S2PLOT device */
  if (!s2opend("/S2FISH", argc, argv)) {
    fprintf(stderr, "Failed to open S2PLOT device - exiting!\n");
    exit(-1);
  }
  s2swin(-1.,1.,-1.,1.,-1.,1.);

  /* for a white bg ... */
  ss2sbc(1., 1., 1.);
  s2scr(1, 0., 0., 0.);

  /* set the camera position to the origin, looking at the pole */
  XYZ position = {0., 0., 0.};
  XYZ up = {1., 0., 0.};
  XYZ vdir = {0., 0., 1.};
  ss2sc(position, up, vdir, 0);

  /* initialise coordinate transformations */
  kinit();

  /* install the callback to draw the sky each frame */
  cs2scb(skycb);

  /* points and axes drawn, so enter interactive display mode */
  s2show(1);

  return 0;
}

void skycb(double *time, int *keycount) {
  static int hr = 0;
  static int mn = 0;

  /* set time and date */
  setut(2005L, 8, 30., (float)hr, mn, 0);
  
  double saz, sel, sra, sdec;
  XYZ posn, norm, start;
  float x, y, z;
  int i;

  /* fetch sun position (az,el) & (ra,dec) (in degrees I hope) */
  coco_sun(&saz, &sel, &sra, &sdec);
  /* draw a yellow circle for the sun at given (az,el) */
  posn.x = posn.y = posn.z = 0.;
  norm.x = cos(sel * D2R) * cos(-saz * D2R);
  norm.y = cos(sel * D2R) * sin(-saz * D2R);
  norm.z = sin(sel * D2R);
  /* radius of Sun on sky is 0.25 degrees */
  start.x = cos((sel+0.25) * D2R) * cos(-saz * D2R);
  start.y = cos((sel+0.25) * D2R) * sin(-saz * D2R);
  start.z = sin((sel+0.25) * D2R);
  s2sci(S2_PG_YELLOW);
  s2slw(2);
  ns2varc(posn, norm, start, 360., 12);

  /* the moon */
  coco_moon(&saz, &sel, &sra, &sdec);
  /* draw a grey circle for the moon at given (az,el) */
  posn.x = posn.y = posn.z = 0.;
  norm.x = cos(sel * D2R) * cos(-saz * D2R);
  norm.y = cos(sel * D2R) * sin(-saz * D2R);
  norm.z = sin(sel * D2R);
  /* radius of Sun on sky is 0.25 degrees */
  start.x = cos((sel+0.25) * D2R) * cos(-saz * D2R);
  start.y = cos((sel+0.25) * D2R) * sin(-saz * D2R);
  start.z = sin((sel+0.25) * D2R);
  s2sci(S2_PG_LTGRAY);
  s2slw(2);
  ns2varc(posn, norm, start, 360., 12);
  
  /* constellation Crux */
  s2slw(2);
  s2sci(S2_PG_WHITE);
  for (i = 0; i < 5; i++) {
    sra = (crux[i][0] + crux[i][1]/60. + crux[i][2]/3600.) * 15.;
    sdec = fabsf(crux[i][4]) + crux[i][5]/60. + crux[i][6]/3600.;
    sdec *= crux[i][3];
    if (!rd2ae(&saz, &sel, sra, sdec)) {
      x = cos(sel * D2R) * cos(-saz * D2R);
      y = cos(sel * D2R) * sin(-saz * D2R);
      z = sin(sel * D2R);
      s2pt1(x, y, z, -2);
    }
  }

  /* lines of elevation */
  float relev = 1.1;
  posn.x = 0.;
  posn.y = 0.;
  posn.z = 0.;
  norm.x = 0.;
  norm.y = 0.;
  norm.z = relev;  
  s2slw(1);
  s2sci(S2_PG_BLUE);
  /* lines of latitude */
  for (i = 0; i < 6; i++) {
    start.x = relev * cos(((float)i * -15.) * D2R);
    start.y = 0.;
    start.z = relev * sin(((float)i * -15.) * D2R);
    ns2varc(posn, norm, start, 360., 90);
    start.z = -start.z;
    ns2varc(posn, norm, start, 360., 90);
  }
  /* lines of azimuth */
  norm.z = 0.;
  start.z = 0.;
  for (i = 0; i < 24; i++) {
    norm.x = -relev * cos(((float)i * 15.) * D2R);
    norm.y = -relev * sin(((float)i * 15.) * D2R);
    start.x = relev * cos(((float)i * 15. + 90.) * D2R);
    start.y = relev * sin(((float)i * 15. + 90.) * D2R);
    ns2varc(posn, norm, start, 360., 45);
  }

  /* celestial latitudes */
  s2sci(S2_PG_GREEN);
  /* fetch pole for normal */
  sra = 0.;
  sdec = 90.;
  if (!rd2ae(&saz, &sel, sra, sdec)) {
    norm.x = cos(sel * D2R) * cos(-saz * D2R);
    norm.y = cos(sel * D2R) * sin(-saz * D2R);
    norm.z = sin(sel * D2R);
    for (i = 0; i < 6; i++) {
      /* fetch ra = 0, dec = X for start vector */
      sdec = (float)i * 15.;
      if (!rd2ae(&saz, &sel, sra, sdec)) {
	start.x = cos(sel * D2R) * cos(-saz * D2R);
	start.y = cos(sel * D2R) * sin(-saz * D2R);
	start.z = sin(sel * D2R);
	ns2varc(posn, norm, start, 360., 90);
      }
      if (!rd2ae(&saz, &sel, sra, -sdec)) {
	start.x = cos(sel * D2R) * cos(-saz * D2R);
	start.y = cos(sel * D2R) * sin(-saz * D2R);
	start.z = sin(sel * D2R);
	ns2varc(posn, norm, start, 360., 90);
      }
    }
  }

  /* celestial longitudes */
  sdec = 0.;
  for (i = 0; i < 12; i++) {
    sra = (float)i * 15.;
    /* fetch start point */
    if (!rd2ae(&saz, &sel, sra, sdec)) {
      start.x = cos(sel * D2R) * cos(-saz * D2R);
      start.y = cos(sel * D2R) * sin(-saz * D2R);
      start.z = sin(sel * D2R);
      /* fetch normal */
      sra += 90.;
      if (!rd2ae(&saz, &sel, sra, sdec)) {
	norm.x = cos(sel * D2R) * cos(-saz * D2R);
	norm.y = cos(sel * D2R) * sin(-saz * D2R);
	norm.z = sin(sel * D2R);
	ns2varc(posn, norm, start, 360., 90);
      }
    }
  }
  
  
  
  

  /* label the time */
  char lab[80];
  sprintf(lab, "\n%02d:%02d", (hr+10)%24, mn);
  s2iden(lab);
  
  /* increase time by X minutes */
  mn += 2;
  if (mn > 59) {
    hr++;
    mn = 0;
  }
  if (hr > 23) {
    hr = 0;
  }

}


#if (0)
  /* loop and plot the points with 2-pixel blocks */
  s2slw(3);
  for (i = 0; i < nvals; i++) {
    if (vel[i] < 0) {
      continue;
    }
    int colidx;
    float x, y, z;

    colidx = MAPLO + (int)((float)(MAPHI - MAPLO) * (loghi[i] - minloghi) /
			   (maxloghi - minloghi) + 0.5);
    if (colidx < MAPLO) {
      colidx = MAPLO;
    }
    if (colidx > MAPHI) {
      colidx = MAPHI;
    }
    s2sci(colidx);

    x = cos(dec[i] * D2R) * cos(ra[i] * D2R);
    y = cos(dec[i] * D2R) * sin(ra[i] * D2R);
    z = sin(dec[i] * D2R);
    if (ss2qpt() < 2) {
      /* "cartesian" projection - place sources at redshift */
      x *= vel[i] / maxvel;
      y *= vel[i] / maxvel;
      z *= vel[i] / maxvel;
    }
    s2pt1(x, y, z, -2);
  }

  /* draw a hemispherical shell to give some context to the data */
  XYZ posn, norm, start;
  posn.x = 0.;
  posn.y = 0.;
  posn.z = 0.;
  norm.x = 0.;
  norm.y = 0.;
  norm.z = 1.;  
  s2slw(1);
  s2sci(S2_PG_DKGREY);
  /* lines of latitude */
  for (i = 0; i < 6; i++) {
    start.x = cos(((float)i * -15.) * D2R);
    start.y = 0.;
    start.z = sin(((float)i * -15.) * D2R);
    ns2varc(posn, norm, start, 360., 90);
    start.z = -start.z;
    ns2varc(posn, norm, start, 360., 90);
  }
  /* lines of longitude */
  norm.z = 0.;
  start.z = 0.;
  float lx[2], ly[2], lz[2];
  lz[0] = lz[1] = 0.;
  for (i = 0; i < 24; i++) {
    norm.x = -cos(((float)i * 15.) * D2R);
    norm.y = -sin(((float)i * 15.) * D2R);
    start.x = cos(((float)i * 15. + 90.) * D2R);
    start.y = sin(((float)i * 15. + 90.) * D2R);
    ns2varc(posn, norm, start, 360., 45);
    lx[0] = start.x;
    lx[1] = -start.x;
    ly[0] = start.y;
    ly[1] = -start.y;
    //s2line(2, lx, ly, lz);
  }

  /* now for testing for dome, I want to draw a circle on the "sky"
   * which is the unit sphere centred on the origin.  Propose doing
   * this by using ns2varc function, with posn = the origin, norm =
   * direction to centre of desired circle, and start = vector to 
   * a point on the circle (and thus on the "sky").
   */
  posn.x = posn.y = posn.z = 0.;
  /* centre of circle is 9h, -25 degrees */
  norm.x = cos(9. * 15. * D2R) * cos(-25. * D2R);
  norm.y = sin(9. * 15. * D2R) * cos(-25. * D2R);
  norm.z = sin(-25. * D2R);
  /* a position on the circle is 9h, -32 degrees for radius = 7 degrees */
  start.x = cos(9. * 15. * D2R) * cos(-32. * D2R);
  start.y = sin(9. * 15. * D2R) * cos(-32. * D2R);
  start.z = sin(-32. * D2R);
  s2sci(S2_PG_CYAN);
  s2slw(2);
  ns2varc(posn, norm, start, 360., 60);


  /* if we are in a dome ... */
  if (ss2qpt() == 2) {
    /* set the camera position to the origin, looking at the pole */
    XYZ position = {0., 0., 0.};
    XYZ up = {0., 1., 0.};
    XYZ vdir = {0., 0., -1.};
    ss2sc(position, up, vdir, 0);

    /* disallow translations of the camera */
    ss2tc(0);

    /* rotate the fisheye to place the pole 52 deg. up from horizon */
    /* (52deg = 90deg - approx latitude of Swinburne) :-) */
    ss2sfra(52.0);

  }

#endif
