#!/bin/csh
#
# texturise1.csh: create texture file having pow-2 side lengths and
#                 being 24-bit RGB colour, from input image.  Just
#                 one texture, with a named output file.
#
# David Barnes, December 2005
#
# $Id: texturise1.csh,v 1.3 2006/10/19 05:40:57 dbarnes Exp $

if (!(${?S2PATH})) then
  echo "S2PATH environment variable MUST be set ... please fix and retry."
  exit(-1);
endif
if (! -d $S2PATH || ! -e ${S2PATH}/s2plot.csh) then
  echo "S2PATH is set but invalid: ${S2PATH} ... please fix and retry."
  exit(-1);
endif
source ${S2PATH}/s2plot.csh
if ($status) then
  exit(-1)
endif

if (!(${?S2PLOT_IMPATH})) then
#  echo "S2PLOT_IMPATH environment variable not set.  Please check your"
#  echo "s2plot.csh file.  Proceeding with default (/usr/bin)."
  setenv S2PLOT_IMPATH /usr/bin
endif
# should check for presence of required binaries...


# usage: "texturise.csh [-crop | -scale | -scale+] inputimage outputimage"
#
# -crop:  crop both sides to largest power-of-2 that is still smaller
#         than each sidelength; image will not be stretched/squashed,
#         but the sides of the image will probably be cropped off.
#
# -scale: scale both sides to largest power-of-2 that is still smaller
#         than each sidelength; the entire image will be present, but
#         will probably be stretched/squashed.
#
# -scale+: scale both sides to largest power-of-2 that is just larger
#          than each sidelength; the entire image will be present, but
#          will probably be stretched/squashed.
#
# ONE OF -crop OR -scale OR -scale+ MUST BE GIVEN!
#
# inputimage: name of input image to convert.  All formats supported 
#             by ImageMagick are allowed.
#
# script *requires* ImageMagick, awk and bc to operate.  Very little
# sanity checking is done.

if ($#argv < 3) then
  echo
  echo "Usage: textureise.csh [-crop | -scale | -scale+] inputimage outputimage"
  echo "Must give one of -crop or -scale or -scale+"
  echo
  exit(-1)
endif

set action=$1
if (("$action" != "-crop") && ("$action" != "-scale") && ("$action" != "-scale+")) then
  echo
  echo "Usage: textureise.csh [-crop | -scale | -scale+] inputimage outputimage"
  echo "Must give one of -crop, -scale or -scale+"
  echo
  exit(-1)
endif

set imp=$S2PLOT_IMPATH

set iname=$argv[2]
set oname=$argv[3]

$imp/identify $iname >& /dev/null
if ($status) then
  echo "  ! cannot find or understand file - $iname - skipping !"
  exit(-1)
endif

# find input image dimensions ix, iy
set ixydim=`$imp/identify $iname | awk '{print $3}'`
set ix=`echo $ixydim | awk -F x '{print $1}'`
set iy=`echo $ixydim | awk -F x '{print $2}'`
set iy=`echo $iy | awk -F + '{print $1}'`

echo $ix $iy

# figure out output image dimensions ox, oy
set ox=1;
while ($ox <= $ix) 
  @ ox = $ox + $ox
end
if ("$action" != "-scale+") then
  set ox=`echo $ox / 2 | bc`
endif
set oy=1;
while ($oy <= $iy) 
  @ oy = $oy + $oy
end
if ("$action" != "-scale+") then
  set oy=`echo $oy / 2 | bc`
endif

# switch based on requested action  
if ("$action" == "-crop") then
  # calculate x and y offsets for centred crop, dx and dy
  set dx=`echo "($ix - $ox) / 2" | bc`
  set dy=`echo "($iy - $oy) / 2" | bc`
  $imp/convert -crop ${ox}x${oy}+${dx}+${dy} -colorspace RGB $iname $oname
else if (("$action" == "-scale") || ("$action" == "-scale+")) then
  set geom="${ox}x${oy}+0+0"
  $imp/convert -resize ${ox}x${oy}! -colorspace RGB -type TrueColor $iname $oname
endif

if ($status == 0) then
  #echo "  > wrote output file $oname"
  #echo $oname
else 
  #echo "  ! failed to convert file !"
  exit(-1)
endif

